MaxRow <- 5000
MinCpgNum <- 5
MetDiff <- 0.25
SdDiff <- 0.2
Top <- 0

suppressMessages(library(getopt))

args = commandArgs(trailingOnly = T)

if (!exists('InFile') || !exists('OutPre')) {
	if (length(args) < 2) {
		message("USAGE: Rscript clustering in_file out_prefix has_feature\n")
		quit(save='no', status=-1)
	}

	InFile = args[1]
	OutPre = args[2]
	Feature = args[3]
	if (length(args) > 4) opt = getopt(matrix(ncol=4, byrow=T, c(
		'diff', 'd', 2, 'numeric',
		'sd', 's', 2, 'numeric',
		'cpg', 'c', 2, 'integer',
		'top', 't', 2, 'integer',
		'group', 'g', 2, 'character'
		)), args[4:length(args)])
	else opt = list()
	if (!is.null(opt$cpg  )) MinCpgNum = as.integer(opt$cpg)
	if (!is.null(opt$diff )) MetDiff   = as.numeric(opt$diff)
	if (!is.null(opt$sd   )) SdDiff    = as.numeric(opt$sd)
	if (!is.null(opt$top  )) Top       = as.numeric(opt$top)
	if (!is.null(opt$group)) GrpFile   = opt$group
}

suppressMessages(library(geneplotter))
suppressMessages(library(gplots))


openGraphics <- function(path, width, height)
{
	flag = F
	if (exists('PdfFlag')) { if (PdfFlag) flag = T }
	if (!flag) { flag = any(grep("\\.pdf$", path)) }

	if (flag)
	{
		if (!any(grep("\\.pdf$", path))) path <- paste(path, ".pdf", sep="")
		pdf(path, w=width/100, h=height/100)
	}
	else
	{
		if (!any(grep("\\.png$", path))) path <- paste(path, ".png", sep="")
		png(path, w=width, h=height)
	}
}

closeGraphics <- function()
{
	graphics.off()
}

loadFile <- function(file, header=T)
{
	print(paste("Loading", file, ".."))
	data <- read.table(file, sep="\t", header=header, check.names=F, comment.char='')
	print(head(data))
	print(tail(data))
	cat("dim=", dim(data), "\n")
	return(data)
}

drawHist <- function(var, file=NA, ...)
{
	if (!is.na(file)) openGraphics(file, 600, 400)
#	r <- range(as.matrix(var))
#	h <- hist(as.matrix(var), breaks=c((r[1]-0.5):(r[2]+0.5)), ...)
	h <- hist(as.matrix(var), breaks=100, ...)
	print(h$counts)
	if (!is.na(file)) closeGraphics()
}

drawHeatmap <- function(x, col=bluered(100), file=NA, ...)
{
	if (!is.na(file)) openGraphics(file, 600, 800)

	hclust2 <- function(x, method="average", ...) hclust(x, method=method, ...)
	dist2 <- function(x, ...) as.dist(1-cor(t(x), method="pearson"))
	if (exists('grpColors')) hv <- heatmap.2(as.matrix(x), distfun=dist2, hclustfun=hclust2, col=col, scale="none", key=TRUE, symkey=FALSE, density.info="density", trace="none", labRow=NA, cexRow=1, margins=c(6,1), lhei=c(1.5,8.5), ColSideColors=grpColors, ...)
	else                      hv <- heatmap.2(as.matrix(x), distfun=dist2, hclustfun=hclust2, col=col, scale="none", key=TRUE, symkey=FALSE, density.info="density", trace="none", labRow=NA, cexRow=1, margins=c(6,1), lhei=c(1.5,8.5), ...)

	if (!is.na(file)) closeGraphics()
	return(hv)
}

analysis <- function(data, suffix)
{
	index <- DataCol+1
	if (nrow(data) <= 1) return()

	hv <- drawHeatmap(data[index:length(data)], Rowv=T, Colv=T, file=paste(OutPre, suffix, 'heatmap', 'png', sep='.'))
	write.table(data[rev(hv$rowInd), c(1:(index-1), index-1+hv$colInd)], file=paste(OutPre, suffix, 'heatmap', 'xls', sep='.'), quote = F, sep = "\t", row.names = F, col.names=T)
}


cat("Start: ", date(), "\n")

PdfFlag = 0
DataCol = ifelse(Feature, 5, 4)

org <- loadFile(InFile)
colnames(org)[1:3] <- c('chr', 'start', 'end')
if (DataCol > 4) colnames(org)[4] <- 'ID'

icpg <- seq(DataCol+0, length(org), 3)
imet <- seq(DataCol+1, length(org), 3)
itot <- seq(DataCol+2, length(org), 3)

dat = na.omit(org)
cat('Removed features with NA: ', nrow(org)-nrow(dat), '\n')

dat$MinCpgNum = apply(dat[DataCol:length(dat)], 1, min)
cat('Removed features with minimum #CpGs of', MinCpgNum, ': ', sum(dat$MinCpgNum < MinCpgNum), '\n')
dat = dat[dat$MinCpgNum >= MinCpgNum,]
if (Feature) dat$ID = factor(dat$ID)
print(dim(dat))

pct = cbind(dat[1:(DataCol-1)], dat[length(dat)], data.frame(t(apply(dat[DataCol:(length(dat)-1)], 1, function(x) x[seq(2,length(x),3)]/x[seq(3,length(x),3)]))))
colnames(pct)[(DataCol+1):(DataCol+length(itot))] = colnames(org)[itot]

if (exists('GrpFile')) {
	grp <- loadFile(GrpFile, header=F)
	colnames(grp) = c('file', 'flag', 'sample', 'group')

#	pct.grp <- apply(as.matrix(as.character(grp$sample)), 1, function(x) grp$group[x==colnames(pct)[(DataCol+1):length(pct)]])
	pct.grp <- apply(as.matrix(colnames(pct[(DataCol+1):length(pct)])), 1, function(x) grp$group[x==grp$sample])
	print(pct.grp)
	stopifnot(length(pct.grp) != 0)
	grpColors <- c('green3', 'gold', 'purple', 'orange', 'magenta', 'cyan')[pct.grp]
}

#for (i in (DataCol+1):length(pct)) {
#	drawHist(na.omit(pct[i]), file=paste(OutPre, 'methy', tolower(colnames(pct)[i]), sep='.'))
#}

	type = 'clust.all'
	print(type)
	cat('Dimension =', dim(pct), '\n')
	try(analysis(pct, type))

	type = paste('clust.delta', MetDiff, sep='')
	print(type)
	pct$dif = apply(pct[(DataCol+1):length(pct)], 1, function(x) {y=range(x); y[2]-y[1]})
	print(range(pct$dif))
	cat('Removed features based on difference between maximum and minimum: ', sum(pct$dif < MetDiff), '\n')
	pct.2 = pct[pct$dif >= MetDiff,]
	cat('Dimension =', dim(pct.2), '\n')
	if (nrow(pct.2) > MaxRow) {
		type = paste('clust.delta', MaxRow, sep='')
		pct.2 = pct[order(pct$dif, decreasing=T),][1:min(nrow(pct), MaxRow),]
	}
	try(analysis(pct.2[1:(length(pct.2)-1)], type))
	if (Top > 0) {
		type = paste('clust.deltatop', Top, sep='')
		pct.2 = pct[order(pct$dif, decreasing=T),][1:min(nrow(pct), Top),]
		try(analysis(pct.2[1:(length(pct.2)-1)], type))
	}
	pct$dif = NULL

	type = paste('clust.sd', SdDiff, sep='')
	print(type)
   pct$sd = apply(pct[(DataCol+1):length(pct)], 1, sd)
	print(range(pct$sd))
	cat('Removed features based on standard deviation: ', sum(pct$sd < SdDiff), '\n')
	pct.3 = pct[pct$sd >= SdDiff,]
	cat('Dimension =', dim(pct.3), '\n')
	if (nrow(pct.3) > MaxRow) {
		type = paste('clust.sd', MaxRow, sep='')
		pct.3 = pct[order(pct$sd, decreasing=T),][1:min(nrow(pct), MaxRow),]
	}
	try(analysis(pct.3[1:(length(pct.3)-1)], type))
	if (Top > 0) {
		type = paste('clust.sdtop', Top, sep='')
		pct.3 = pct[order(pct$sd, decreasing=T),][1:min(nrow(pct), Top),]
		try(analysis(pct.3[1:(length(pct.3)-1)], type))
	}
	pct$sd = NULL

cat("End: ", date(), "\n")
